@@ -105,9 +105,8 @@ import Test.HUnit
 
 import Hledger
 import Prelude hiding (putStr)
-import Hledger.Utils.UTF8 (putStr)
-import Hledger.Cli.Format
-import qualified Hledger.Cli.Format as Format
+import Hledger.Utils.UTF8IOCompat (putStr)
+import Hledger.Data.FormatStrings
 import Hledger.Cli.Options
 
 
@@ -117,7 +116,7 @@ balance CliOpts{reportopts_=ropts} j = do
   d <- getCurrentDay
   let lines = case formatFromOpts ropts of
             Left err -> [err]
-            Right _ -> accountsReportAsText ropts $ accountsReport ropts (optsToFilterSpec ropts d) j
+            Right _ -> accountsReportAsText ropts $ accountsReport ropts (queryFromOpts d ropts) j
   putStr $ unlines lines
 
 -- | Render a balance report as plain text suitable for console output.
@@ -134,6 +133,20 @@ accountsReportAsText opts (items, total) = concat lines ++ t
                 ,padleft 20 $ showMixedAmountWithoutPrice total
                 ]
 
+tests_accountsReportAsText = [
+  "accountsReportAsText" ~: do
+  -- "unicode in balance layout" ~: do
+    j <- readJournal'
+      "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
+    let opts = defreportopts
+    accountsReportAsText opts (accountsReport opts (queryFromOpts (parsedate "2008/11/26") opts) j) `is`
+      ["                -100  актив:наличные"
+      ,"                 100  расходы:покупки"
+      ,"--------------------"
+      ,"                   0"
+      ]
+ ]
+
 {-
 This implementation turned out to be a bit convoluted but implements the following algorithm for formatting:
 
@@ -170,15 +183,14 @@ formatAccountsReportItem opts accountName depth amount (fmt:fmts) =
          FormatLiteral l -> l
          FormatField ljust min max field  -> formatField opts accountName depth amount ljust min max field
 
-formatField :: ReportOpts -> Maybe AccountName -> Int -> Amount -> Bool -> Maybe Int -> Maybe Int -> Field -> String
+formatField :: ReportOpts -> Maybe AccountName -> Int -> Amount -> Bool -> Maybe Int -> Maybe Int -> HledgerFormatField -> String
 formatField opts accountName depth total ljust min max field = case field of
-        Format.Account     -> formatValue ljust min max $ maybe "" (accountNameDrop (drop_ opts)) accountName
-        Format.DepthSpacer -> case min of
+        AccountField     -> formatValue ljust min max $ maybe "" (accountNameDrop (drop_ opts)) accountName
+        DepthSpacerField -> case min of
                                Just m  -> formatValue ljust Nothing max $ replicate (depth * m) ' '
                                Nothing -> formatValue ljust Nothing max $ replicate depth ' '
-        Format.Total       -> formatValue ljust min max $ showAmountWithoutPrice total
+        TotalField       -> formatValue ljust min max $ showAmountWithoutPrice total
         _                  -> ""
 
 tests_Hledger_Cli_Balance = TestList
- [
- ]
+  tests_accountsReportAsText
