@@ -1,4 +1,4 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskell, ScopedTypeVariables #-}
 {-|
 
 Command-line options for the hledger program, and option-parsing utilities.
@@ -7,6 +7,7 @@ Command-line options for the hledger program, and option-parsing utilities.
 
 module Hledger.Cli.Options
 where
+import Control.Exception as C
 import Data.List
 import Data.List.Split
 import Data.Maybe
@@ -22,12 +23,12 @@ import Text.ParserCombinators.Parsec
 import Text.Printf
 
 import Hledger
-import Hledger.Cli.Format as Format
+import Hledger.Data.FormatStrings as Format
 import Hledger.Cli.Version
 
 
 -- 1. cmdargs mode and flag definitions, for the main and subcommand modes.
--- Flag values are parsed initially to simple RawOpts to permit reuse.
+-- Flag values are parsed initially to a simple association list to allow reuse.
 
 type RawOpts = [(String,String)]
 
@@ -59,11 +60,11 @@ mainmode addons = defmode {
      groupUnnamed = [
      ]
     ,groupHidden = [
+        convertmode
      ]
     ,groupNamed = [
       ("Misc commands", [
         addmode
-       ,convertmode
        ,testmode
        ])
      ,("\nReport commands", [
@@ -72,6 +73,9 @@ mainmode addons = defmode {
        ,postingsmode
        -- ,transactionsmode
        ,activitymode
+       ,incomestatementmode
+       ,balancesheetmode
+       ,cashflowmode
        ,statsmode
        ])
      ]
@@ -80,6 +84,19 @@ mainmode addons = defmode {
     }
  }
 
+-- backwards compatibility - allow cmdargs to recognise this command so we can detect and warn
+convertmode = (commandmode ["convert"]) {
+  modeValue = [("command","convert")]
+ ,modeHelp = ""
+ ,modeArgs = ([], Just $ flagArg (\s opts -> Right $ setopt "args" s opts) "[CSVFILE]")
+ ,modeGroupFlags = Group {
+     groupUnnamed = []
+    ,groupHidden = []
+    ,groupNamed = []
+    }
+ }
+--
+
 addonmode name = defmode {
   modeNames = [name]
  ,modeHelp = printf "[-- OPTIONS]   run the %s-%s program" progname name
@@ -107,6 +124,7 @@ generalflags3 = helpflags
 
 fileflags = [
   flagReq ["file","f"]  (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different journal file; - means stdin"
+ ,flagReq ["rules-file"]  (\s opts -> Right $ setopt "rules-file" s opts) "RULESFILE" "conversion rules for CSV (default: FILE.rules)"
  ,flagReq ["alias"]  (\s opts -> Right $ setopt "alias" s opts)  "ACCT=ALIAS" "display ACCT's name as ALIAS in reports"
  ]
 
@@ -154,19 +172,6 @@ addmode = (commandmode ["add"]) {
     }
  }
 
-convertmode = (commandmode ["convert"]) {
-  modeValue = [("command","convert")]
- ,modeHelp = "show the specified CSV file as hledger journal entries"
- ,modeArgs = ([], Just $ flagArg (\s opts -> Right $ setopt "args" s opts) "[CSVFILE]")
- ,modeGroupFlags = Group {
-     groupUnnamed = [
-      flagReq ["rules-file"]  (\s opts -> Right $ setopt "rules-file" s opts) "FILE" "rules file to use (default: CSVFILE.rules)"
-     ]
-    ,groupHidden = []
-    ,groupNamed = [(generalflagstitle, generalflags3)]
-    }
- }
-
 testmode = (commandmode ["test"]) {
   modeHelp = "run self-tests, or just the ones matching REGEXPS"
  ,modeArgs = ([], Just $ flagArg (\s opts -> Right $ setopt "args" s opts) "[REGEXPS]")
@@ -177,7 +182,7 @@ testmode = (commandmode ["test"]) {
     }
  }
 
-accountsmode = (commandmode ["balance","accounts"]) {
+accountsmode = (commandmode ["balance","bal","accounts"]) {
   modeHelp = "(or accounts) show matched accounts and their balances"
  ,modeArgs = ([], Just commandargsflag)
  ,modeGroupFlags = Group {
@@ -234,6 +239,36 @@ activitymode = (commandmode ["activity","histogram"]) {
     }
  }
 
+incomestatementmode = (commandmode ["incomestatement","is"]) {
+  modeHelp = "show a standard income statement"
+ ,modeArgs = ([], Just commandargsflag)
+ ,modeGroupFlags = Group {
+     groupUnnamed = []
+    ,groupHidden = []
+    ,groupNamed = [(generalflagstitle, generalflags1)]
+    }
+ }
+
+balancesheetmode = (commandmode ["balancesheet","bs"]) {
+  modeHelp = "show a standard balance sheet"
+ ,modeArgs = ([], Just commandargsflag)
+ ,modeGroupFlags = Group {
+     groupUnnamed = []
+    ,groupHidden = []
+    ,groupNamed = [(generalflagstitle, generalflags1)]
+    }
+ }
+
+cashflowmode = (commandmode ["cashflow","cf"]) {
+  modeHelp = "show a simple cashflow statement"
+ ,modeArgs = ([], Just commandargsflag)
+ ,modeGroupFlags = Group {
+     groupUnnamed = []
+    ,groupHidden = []
+    ,groupNamed = [(generalflagstitle, generalflags1)]
+    }
+ }
+
 statsmode = (commandmode ["stats"]) {
   modeHelp = "show quick statistics for a journal (or part of it)"
  ,modeArgs = ([], Just commandargsflag)
@@ -253,10 +288,10 @@ data CliOpts = CliOpts {
      rawopts_         :: RawOpts
     ,command_         :: String
     ,file_            :: Maybe FilePath
+    ,rules_file_      :: Maybe FilePath
     ,alias_           :: [String]
     ,debug_           :: Bool
     ,no_new_accounts_ :: Bool           -- add
-    ,rules_file_      :: Maybe FilePath -- convert
     ,reportopts_      :: ReportOpts
  } deriving (Show)
 
@@ -282,10 +317,10 @@ toCliOpts rawopts = do
               rawopts_         = rawopts
              ,command_         = stringopt "command" rawopts
              ,file_            = maybestringopt "file" rawopts
-             ,alias_           = listofstringopt "alias" rawopts
+             ,rules_file_      = maybestringopt "rules-file" rawopts
+             ,alias_           = map stripquotes $ listofstringopt "alias" rawopts
              ,debug_           = boolopt "debug" rawopts
              ,no_new_accounts_ = boolopt "no-new-accounts" rawopts -- add
-             ,rules_file_      = maybestringopt "rules-file" rawopts -- convert
              ,reportopts_ = defreportopts {
                              begin_     = maybesmartdateopt d "begin" rawopts
                             ,end_       = maybesmartdateopt d "end" rawopts
@@ -308,7 +343,7 @@ toCliOpts rawopts = do
                             ,quarterly_ = boolopt "quarterly" rawopts
                             ,yearly_    = boolopt "yearly" rawopts
                             ,format_    = maybestringopt "format" rawopts
-                            ,patterns_  = listofstringopt "args" rawopts
+                            ,query_     = unwords $ listofstringopt "args" rawopts
                             }
              }
 
@@ -336,11 +371,11 @@ getHledgerProgramsInPath = do
     where
       hledgerprog = string progname >> char '-' >> many1 (letter <|> char '-') >> eof
 
-getEnvSafe v = getEnv v `catch` (\_ -> return "")
-getDirectoryContentsSafe d = getDirectoryContents d `catch` (\_ -> return [])
+getEnvSafe v = getEnv v `C.catch` (\(_::C.IOException) -> return "")
+getDirectoryContentsSafe d = getDirectoryContents d `C.catch` (\(_::C.IOException) -> return [])
 
 -- | Convert possibly encoded option values to regular unicode strings.
-decodeRawOpts = map (\(name,val) -> (name, fromPlatformString val))
+decodeRawOpts = map (\(name,val) -> (name, fromSystemString val))
 
 -- A hacky workaround for http://code.google.com/p/ndmitchell/issues/detail?id=470 :
 -- we'd like to permit options before COMMAND as well as after it.
@@ -364,7 +399,7 @@ maybestringopt name = maybe Nothing (Just . stripquotes) . lookup name
 
 stringopt name = fromMaybe "" . maybestringopt name
 
-listofstringopt name rawopts = [stripquotes v | (n,v) <- rawopts, n==name]
+listofstringopt name rawopts = [v | (k,v) <- rawopts, k==name]
 
 maybeintopt :: String -> RawOpts -> Maybe Int
 maybeintopt name rawopts =
@@ -415,21 +450,24 @@ formatFromOpts = maybe (Right defaultBalanceFormatString) parseFormatString . fo
 -- | Default line format for balance report: "%20(total)  %2(depth_spacer)%-(account)"
 defaultBalanceFormatString :: [FormatString]
 defaultBalanceFormatString = [
-      FormatField False (Just 20) Nothing Total
+      FormatField False (Just 20) Nothing TotalField
     , FormatLiteral "  "
-    , FormatField True (Just 2) Nothing DepthSpacer
-    , FormatField True Nothing Nothing Format.Account
+    , FormatField True (Just 2) Nothing DepthSpacerField
+    , FormatField True Nothing Nothing AccountField
     ]
 
--- | Get the journal file path from options, an environment variable, or a default.
--- If the path contains a literal tilde raise an error to avoid confusion.
+-- | Get the (tilde-expanded, absolute) journal file path from options, an environment variable, or a default.
 journalFilePathFromOpts :: CliOpts -> IO String
 journalFilePathFromOpts opts = do
-  f <- myJournalPath
-  let f' = fromMaybe f $ file_ opts
-  if '~' `elem` f'
-   then error' $ printf "~ in the journal file path is not supported, please adjust (%s)" f'
-   else return f'
+  f <- defaultJournalPath
+  d <- getCurrentDirectory
+  expandPath d $ fromMaybe f $ file_ opts
+
+-- | Get the (tilde-expanded) rules file path from options, if any.
+rulesFilePathFromOpts :: CliOpts -> IO (Maybe FilePath)
+rulesFilePathFromOpts opts = do
+  d <- getCurrentDirectory
+  maybe (return Nothing) (fmap Just . expandPath d) $ rules_file_ opts
 
 aliasesFromOpts :: CliOpts -> [(AccountName,AccountName)]
 aliasesFromOpts = map parseAlias . alias_
