@@ -1,3 +1,4 @@
+{-# LANGUAGE ScopedTypeVariables #-}
 {-|
 
 Utilities for top-level modules and ghci. See also Hledger.Read and
@@ -8,7 +9,6 @@ Hledger.Utils.
 module Hledger.Cli.Utils
     (
      withJournalDo,
-     readJournal',
      journalReload,
      journalReloadIfChanged,
      journalFileIsNewer,
@@ -21,7 +21,7 @@ module Hledger.Cli.Utils
      Test(TestList),
     )
 where
-import Control.Exception
+import Control.Exception as C
 import Data.List
 import Data.Maybe
 import Safe (readMay)
@@ -48,20 +48,18 @@ withJournalDo opts cmd = do
   -- We kludgily read the file before parsing to grab the full text, unless
   -- it's stdin, or it doesn't exist and we are adding. We read it strictly
   -- to let the add command work.
-  journalFilePathFromOpts opts >>= readJournalFile Nothing >>=
-    either error' (cmd opts . journalApplyAliases (aliasesFromOpts opts))
+  rulespath <- rulesFilePathFromOpts opts
+  journalpath <- journalFilePathFromOpts opts
+  ej <- readJournalFile Nothing rulespath journalpath
+  either error' (cmd opts . journalApplyAliases (aliasesFromOpts opts)) ej
 
 -- -- | Get a journal from the given string and options, or throw an error.
 -- readJournalWithOpts :: CliOpts -> String -> IO Journal
--- readJournalWithOpts opts s = readJournal Nothing s >>= either error' return
-
--- | Get a journal from the given string, or throw an error.
-readJournal' :: String -> IO Journal
-readJournal' s = readJournal Nothing s >>= either error' return
+-- readJournalWithOpts opts s = readJournal Nothing Nothing Nothing s >>= either error' return
 
 -- | Re-read a journal from its data file, or return an error string.
 journalReload :: Journal -> IO (Either String Journal)
-journalReload j = readJournalFile Nothing $ journalFilePath j
+journalReload j = readJournalFile Nothing Nothing $ journalFilePath j
 
 -- | Re-read a journal from its data file mostly, only if the file has
 -- changed since last read (or if there is no file, ie data read from
@@ -100,7 +98,7 @@ journalSpecifiedFileIsNewer Journal{filereadtime=tread} f = do
 fileModificationTime :: FilePath -> IO ClockTime
 fileModificationTime f
     | null f = getClockTime
-    | otherwise = getModificationTime f `Prelude.catch` \_ -> getClockTime
+    | otherwise = getModificationTime f `C.catch` \(_::C.IOException) -> getClockTime
 
 -- | Attempt to open a web browser on the given url, all platforms.
 openBrowserOn :: String -> IO ExitCode
@@ -140,7 +138,7 @@ writeFileWithBackup :: FilePath -> String -> IO ()
 writeFileWithBackup f t = backUpFile f >> writeFile f t
 
 readFileStrictly :: FilePath -> IO String
-readFileStrictly f = readFile f >>= \s -> Control.Exception.evaluate (length s) >> return s
+readFileStrictly f = readFile f >>= \s -> C.evaluate (length s) >> return s
 
 -- | Back up this file with a (incrementing) numbered suffix, or give an error.
 backUpFile :: FilePath -> IO ()
